package com.swagger.rest.controllers

import com.swagger.rest.models.MemberInput
import com.swagger.rest.models.ProjectMember
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("/api/v1")
class MemberController(
    private val memberRepository: MemberRepository,
    private val projectRepository: ProjectRepository,
    private val userRepository: UserRepository
) {

    @GetMapping("/projects/{id}/member")
    fun getMemberByProject(@PathVariable("id") id: Long): Any {
        val memberData = memberRepository.findByProject(id.toString())
        return if (memberData.isNotEmpty()) {
            ResponseEntity<List<ProjectMember>>(memberData, HttpStatus.OK)
        } else {
            ResponseEntity<List<ProjectMember>>(HttpStatus.NOT_FOUND)
        }
    }

    @GetMapping("/projects/member/{id}")
    fun getMemberProject(@PathVariable("id") id: Long): Any {
        val memberData = memberRepository.findById(id)
        return if (memberData.isPresent) {
            ResponseEntity<ProjectMember>(memberData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<ProjectMember>(HttpStatus.NOT_FOUND)
        }
    }

    @DeleteMapping("/projects/member/{id}")
    fun deleteProjectMember(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        return try {
            val find = memberRepository.findById(id)
            if (find.isPresent) {
                val validOwner =
                    projectRepository.validOwner(find.get().project_id!!.id.toString(), userId.id.toString())
                val validAdmin =
                    memberRepository.validRole(find.get().project_id!!.id.toString(), userId.id.toString(), "2")
                val targetNotAdmin = memberRepository.targetRole(
                    find.get().project_id!!.id.toString(),
                    find.get().user_id!!.id.toString(),
                    find.get().role.toString()
                )
                if (validOwner > 0 || (validAdmin > 0 && targetNotAdmin > 0)) {
                    memberRepository.deleteById(id)
                    ResponseEntity(HttpStatus.OK)
                } else {
                    ResponseEntity(HttpStatus.FORBIDDEN)
                }
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PostMapping("/projects/{id}/member")
    fun addProjectMember(
        @PathVariable("id") id: Long, @RequestBody member: MemberInput
    ): Any {
        return try {
            val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            val found = projectRepository.findById(id)
            val user = userRepository.findById(member.user_id!!.toLong())
            val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
            val validAdmin = memberRepository.validRole(id.toString(), userId.id.toString(), "2")
            if (validOwner > 0 || validAdmin > 0) {
                if (enumValues<Member>().any { it.name == member.role!!.uppercase() }) {
                    val role = Member.valueOf(member.role!!.uppercase()).ordinal
                    val dup = memberRepository.findDup(id.toString(), role.toString(), member.user_id.toString()).size
                    if (found.isEmpty || user.isEmpty) {
                        ResponseEntity<ProjectMember>(HttpStatus.NOT_FOUND)
                    } else if (dup > 0) {//duplicate
                        ResponseEntity<ProjectMember>(HttpStatus.CONFLICT)
                    } else {
                        val saveMember = ProjectMember()
                        saveMember.project_id = found.get()
                        saveMember.user_id = user.get()
                        saveMember.role = role
                        ResponseEntity<ProjectMember>(memberRepository.save(saveMember), HttpStatus.CREATED)
                    }
                } else {// invalid
                    ResponseEntity<ProjectMember>(HttpStatus.BAD_REQUEST)
                }
            } else {
                ResponseEntity<ProjectMember>(HttpStatus.FORBIDDEN)
            }
        } catch (e: Exception) {
            ResponseEntity<ProjectMember>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }
}

private enum class Member {
    QC, PROGRAMMER, ADMIN
}